// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
//	Test the coordinate conversion chain
//		CHIPCOORD->TOREAL->CAMCOORD1->CAMCOORD2
//
#include <fstream>
#include <iostream>
#include <cmath>
#include "CalServer.h"
#include "Xmm.h"
#include "GeometryDataServer.h"
#include "common.h"

using namespace std;

int main()
{
	for (unsigned i = 0; i < xmmPayLoad.instruments().size(); ++i) {

		XmmInstrument *ins = xmmPayLoad.instruments()[i];
		if (ins->id() == Xmm::ERM)
			continue;

		cerr<<"Instrument: "<<Xmm::toString(ins->id())<<endl;
		calServer.setInstrument(ins->id());
		calServer.state()->randomize()->set(false);
		if (calServer.state()->node()->isActive())
			calServer.state()->node()->set(Xmm::PRIMARY_NODE);
		if (calServer.state()->ocb()->isActive())
			calServer.state()->ocb()->set(1);

		for (unsigned ccd = 1; ccd<=ins->nCcds(); ++ccd) {
			calServer.state()->ccd()->set(ccd);

			int16 x = 1, y = 1;
			real64 dx = .1f, dy = .1,
				   xmm, ymm, zmm;
			CalInt16Vector xv(&x, 1), yv(&y, 1);
			CalReal64Vector dxv(&dx, 1), dyv(&dy, 1);

			GeometryDataServer *cdf = 0; cdf = calServer.getAtom(cdf);
			XmmRealPixelList pl = cdf->chipCoordList(xv, yv).toReal();

			unsigned n=0;
			while (n<21) {
				XmmCartesianPointList cpl =
						pl.toCamCoord1(&xmm, &ymm, &zmm).toCamCoord2();

				cout<<pl.xr(0)<<"/"<<pl.yr(0)<<" = "
					<<xmm<<"/"<<ymm<<"/"<<zmm<<endl;
				pl.add(dxv, dyv);
				++n;
			}
		}
	}
	return 0;
}
