// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//
//	test the AbsorptionDataServer class
//
#include <iostream>
#include <cassert>
#include "CalServer.h"
#include "Ccf.h"
#include "FWCDataServer.h"

using namespace std;

/*
namespace {
bool isClose(double a, double b)
{
    return fabs(a)/fabs(b)-1. < 1.e-5;
}
}
*/

int main()
{
	//	MOS-1
	const Xmm::Instrument instrument = Xmm::EMOS1;
	calServer.setInstrument(instrument);
	calServer.state()->ccd()->set(1);

	//	get the FWCDataServer
	FWCDataServer *fds = 0; fds = calServer.getAtom(fds);

        // Read in the expos time v rev structure
        FWCexptime exp = fds->exptime();

        cout<<"exp [0] "<<exp.rev[0]
            <<" tstart: "<<exp.tstart[0]
            <<" tstop: "<<exp.tstop[0]
            <<" expos: "<<exp.exptime[0]
            <<" ccd: "<<exp.ccd[0]
            <<" nflag: "<<exp.nflag[0]<<endl;

        // Get events for a set of Rev,ccd pairs
        multimap<int, unsigned> revCcd;
        revCcd.insert(make_pair(230,1));
        revCcd.insert(make_pair(230,2));
        vector<FWCevent> fev = fds->getEvents(revCcd);

        cout<<"Number of rows: "<<fev.size()<<endl;

        cout<<"Event 1: "<<fev[0].flag()<<" "<<fev[0].pi()<<endl;
        cout<<"Event 1: "<<fev[0].pattern()<<" "<<fev[0].weight()<<endl;
        cout<<"Event 2: "<<fev[1].rawx()<<" "<<fev[1].rawy()<<endl;
        cout<<"Event 2: "<<fev[1].detx()<<" "<<fev[1].dety()<<endl;
        cout<<"Event 3: "<<fev[2].time()<<" "<<fev[2].ccd()<<endl;
        cout<<"Event 3: "<<fev[2].rev()<<" "<<fev[2].obsid()<<endl;
        cout<<"Event 3: "<<fev[2].expid()<<" "<<fev[2].ccdnoise()<<endl;

//        calServer.state()->mode()->set(Xmm::PRIME_FULL_WINDOW);
        //vector<double> coeffs2 = fds->getDLCoeffs(Xmm::PRIME_FULL_WINDOW);

        //        EPN     

        // Discarded Lines
	calServer.releaseAtom(fds);
	calServer.setInstrument(Xmm::EPN);
//        calServer.state()->mode()->set(Xmm::PRIME_FULL_WINDOW_EXTENDED);

	FWCDataServer *fds2 = 0; fds2 = calServer.getAtom(fds2);
        vector<double> coeffs = fds2->getDLCoeffs(Xmm::PRIME_FULL_WINDOW);
        cout<<"Coeffs: "<<coeffs[0]<<"  "<<coeffs[1]<<endl;
        coeffs = fds2->getDLCoeffs(1);
        cout<<"Coeffs: "<<coeffs[0]<<"  "<<coeffs[1]<<endl;
        //cout<<"Coeffs: "<<coeffs[2]<<"  "<<coeffs[3]<<endl;
        //Replace above when CCF fully ready
        //
        // Get coeffs when mode not given on command line
        calServer.state()->mode()->set(Xmm::PRIME_FULL_WINDOW);
        coeffs = fds2->getDLCoeffs();
        cout<<"Coeffs not given: "<<coeffs[0]<<"  "<<coeffs[1]<<endl;

        // Read in the expos time v rev structure
        exp = fds2->exptime();

        cout<<"exp [0] "<<exp.rev[0]
            <<" tstart: "<<exp.tstart[0]
            <<" tstop: "<<exp.tstop[0]
            <<" expos: "<<exp.exptime[0]
            <<" ccd: "<<exp.ccd[0]
            <<" nflag: "<<exp.nflag[0]<<endl;

        // eFF mode
        calServer.releaseAtom(fds2);
        calServer.state()->mode()->set(Xmm::PRIME_FULL_WINDOW_EXTENDED);
        FWCDataServer *fds3 = 0; fds3 = calServer.getAtom(fds3);

        // Read in the expos time v rev structure
        exp = fds3->exptime();

        cout<<"exp [0] "<<exp.rev[0]
            <<" tstart: "<<exp.tstart[0]
            <<" tstop: "<<exp.tstop[0]
            <<" expos: "<<exp.exptime[0]
            <<" ccd: "<<exp.ccd[0]
            <<" nflag: "<<exp.nflag[0]<<endl;

        Xmm::Mode mi = calServer.state()->mode()->get();
        unsigned short modeIndex = calServer.instrument()->modeIndex(mi);
        coeffs = fds3->getDLCoeffs();
        cout<<"Coeffs EFF: "<<coeffs[0]<<"  "<<coeffs[1]<<endl;
        coeffs = fds3->getDLCoeffs(modeIndex);
        cout<<"Coeffs mi EFF: "<<coeffs[0]<<"  "<<coeffs[1]<<endl;

	return 0;
}







