// ESA (C) 2000-2018 
//
// This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
//
//    SAS is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    SAS is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	RgsSolarAspectAngServer.h --- class related to solar aspect angle coeffs
//
//	DESCRIPTION:
//
//	CHANGE HISTORY:
//		RDS, 2006-03-31: 1.0.0: - first version from BadPixelDataServer.h
//
//	AUTHOR:
//		R. Saxton
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//
//	$Header: /sasbuild/sas_git_repo/cvs_copy/sasdev/sas/packages/cal/src/RgsSolarAspectAngServer.h,v 1.2 2013/02/27 16:24:50 rsaxton Exp $
//

#ifndef RGS_SOLARASPECTANGLE_SERVER_H
#define RGS_SOLARASPECTANGLE_SERVER_H

#include "Ccf.h"
#include "CalAtomsBase.h"

#include <vector>

class RgsSolarAspectAngServer : public CalAtom,
	public virtual CalChild<CalAtomCollection<RgsSolarAspectAngServer> > {
public:
	RgsSolarAspectAngServer(vector<double> saacoeff_order1,
                                vector<double> saacoeff_order2,
					   CalAtomCollection<RgsSolarAspectAngServer> *parent = 0);
		//
		//	PURPOSE:
		//		Instantiate rgs solar aspect angle object
		//


	vector<double> coefficients(unsigned order) const;
		//
		//	PURPOSE:
		//		Return solar aspect angle coefficients
		//

        virtual void dump(ostream &) {}
                //
                //      PURPOSE:
                //              Dump data members to an ostream
                //
                //      ARGUMENTS:
                //              In:
                //                      os:     stream to print on
                //


        static unsigned id() { return Ccf::SAACorr; }
                //
                //      PURPOSE:
                //              Return ID of CalAtom
                //

	~RgsSolarAspectAngServer();
		//
		//	PURPOSE:
		//		Destroy object constructed with above constructor(s).
		//

private:
	//friend class CalAtomCollection<RgsSolarAspectAngServer>;

	std::vector<double>	_saacoeff_order1, _saacoeff_order2;
};

#endif

